/**********************************************************************
 *HY16F188
 * main.c
 * Created on:2013/12/08
 * Maintenance:2014/10/01
 * --------------------------------------------------------------------
 * Release 1.0.0
 * Program Description:
 * --------------------
 *
 * ---------
 *          |              --------------------
 *    PT2.0 | SDA ---> SDA | LCD Drive HY2613 |
 *    PT2.1 | SCK ---> SCK --------------------
 *     GND  |
 *          |
 * ---------
 **********************************************************************/
/*----------------------------------------------------------------------------*/
/* Includes                                                                   */
/*----------------------------------------------------------------------------*/
#include "HY16F188.h"
#include "System.h"
#include "DrvGPIO.h"
#include "DrvI2C.h"
#include "DrvHWI2C.h"
#include "DrvCLOCK.h"
#include "HY2613.h"
#include "my define.h"
#include "DrvREG32.h"
#include "DrvTimer.h"
#include "DrvRTC.h"


/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;

/*----------------------------------------------------------------------------*/
/* DEFINITIONS                                                                */
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/
MCUSTATUS  MCUSTATUSbits;
extern unsigned char seg[16];
#define  Disable 0
#define  Enable  1
unsigned int sec,min,hour,week,day,month,year;	//ɶ
unsigned int b,c,pb6,pb7;						//
unsigned int mm,ss,sss,stopsss,stopss,stopmm;	//X
unsigned int setsec,setmin,sethour,setweek,setday,setmonth,setyear,setnum;		//
unsigned int act,acs,acm,ach,ac;
unsigned int first;
/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/
void ClearLCDframe(void);
void LCD_DATA_DISPLAY_1(unsigned int LcdBuffer);
void LCD_DATA_DISPLAY_2(unsigned int LcdBuffer);
void LCD_DATA_DISPLAY_3(unsigned int LcdBuffer);
void LCD_DATA_DISPLAY_4(unsigned int LcdBuffer);
void LCD_DATA_DISPLAY_5(unsigned int LcdBuffer);
void Delay(unsigned int num);
void LCD_DISPLAY_HYCON(void);
void LCD_DISPLAY_PASS01(void);
void Initial_Array(void);
void RTC_Initial(void);
void Hc(void);
void loopday(void);
void Pb_7();
void Pb_6();
void timeloop(void);
void lookac(void);
void settime(void);
void setac(void);

/*----------------------------------------------------------------------------*/
/* Main Function                                                              */
/*----------------------------------------------------------------------------*/
int main(void)
{
  //unsigned int m;

  DrvCLOCK_EnableHighOSC(E_INTERNAL,50);       // Select HAO 4MHz
  InitalI2C();
  SYS_EnableGIE(7,0x3F);  // Enable GIE(Global Interrupt)

  Ini_Display();
  ClearLCDframe();
  Delay(10000);
  DisplayHYcon();
  Delay(50000);
  MCUSTATUSbits._byte = 0;
  LCD_DATA_DISPLAY(123456);
  int_04=0xff20ff00;
  RTC_Initial();

  pio_00=0xff00ff00;
  pio_04=0xff00ff00;
  pio_08=0x00;
  pio_10=0xff00ff03;
  pio_14=0xff00ff03;

  DrvGPIO_Open(E_PT1,0xc0,E_IO_INPUT);
  DrvGPIO_Open(E_PT1,0xc0,E_IO_PullHigh); //enable PT1_5-7 pull high R
  DrvGPIO_Open(E_PT1,0x10,E_IO_OUTPUT);


  SYS_EnableGIE(7,0X3F);                         //Enable GIE (Global Interrupt Enable)
  DrvGPIO_ClearIntFlag(E_PT1,0xc0);         //clear PT1.4~7 interrupt flag
  DrvCLOCK_EnableHighOSC(E_INTERNAL,50);       //select hsrc 2MHz
  DrvGPIO_ClkGenerator(E_HS_CK,1);          //GPIO CLK enable
  DrvGPIO_Open(E_PT1,0xc0,E_IO_IntEnable);  //PT1.4~7 interrupt enable
  DrvGPIO_IntTrigger(E_PT1,0xc0,E_N_Edge);  //PT1.4~7 interrupt  trigger method is negative edge


  DrvGPIO_Open(E_PT1,0x10,E_IO_OUTPUT);


  S_DRVRTC_TIME_DATA_T sCurTime;//read
  DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
  sec=sCurTime.u32cSecond;
  min=sCurTime.u32cMinute;
  hour=sCurTime.u32cHour;
  week=sCurTime.u32cDayOfWeek;
  day=sCurTime.u32cDay;
  month=sCurTime.u32cMonth;
  year=sCurTime.u32Year;
  asm("NOP");

  LCD_DATA_DISPLAY_1(hour);
  LCD_DATA_DISPLAY_2(min);
  LCD_DATA_DISPLAY_3(sec);

  ac=0;
  acs=00;
  ach=16;
  acm=05;


  while(1)
  {
	  asm volatile("sethi $r0, 0xc0000");
	  asm volatile("ori  $r0, $r0, 0x003f");
	  asm volatile("mtsr $r0, $INT_MASK");
	  asm volatile("movi $r0, 0x70009");
	  asm volatile("mtsr $r0, $PSW");
	  clk_08=0x0101;
	  clk_00=0x0100;
	  sys_04=0x1010;
	  asm("standby 1");
	  lookac();
	  if(b==0) Pb_6();
	  if(pb6==1)
	  {
		  pb6=0;
		  clk_00=0x0f05;
		  clk_08=0x0f00;
		  DrvRTC_PeriodicTimeEnable(0);//set 1/128
		  Hc();
		  DrvRTC_PeriodicTimeEnable(7);//set 1/128
		  clk_08=0x0101;
		  clk_00=0x0100;
	  }
	  if(c==0) Pb_7();
	  if(pb7==1)
	  {
		  pb7=0;
		  clk_00=0x0f05;
		  clk_08=0x0f00;
		  loopday();
		  clk_08=0x0101;
		  clk_00=0x0100;
	  }
	  if(sss>=1)
	  {
		  sss=0;
		  timeloop();
	  }
  }

  return 0;
}


/*---------------------------------------------------------------------------*/
/* Hardware Communication Interrupt                                          */
/* UART/SPI/I2C Interrupt Service Routines                                   */
/*---------------------------------------------------------------------------*/
void HW0_ISR(void)
{
  unsigned char I2C_Status;

  if(DrvI2C_ReadIntFlag()==E_DRVI2C_INT)  // Get I2C Interrupt Flag
  {
    I2C_Status=DrvI2C_GetStatusFlag();  // Get I2C Status Flag
    switch(I2C_Status)
    {
      case 0x90: //MACTFlag+RWFlag
          { /* START has been transmitted */
            DrvI2C_WriteData(I2C_TARGET);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x84: //MACTFlag+ACKFlag
          { /* Slave A + W has been transmitted. ACK has been received. */
            DrvI2C_WriteData(Sendbuf[DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x80: //MACTFlag
          { /* Slave A + W has been transmitted. ACK has been received. */
            DrvI2C_WriteData(Sendbuf[DataTxIndex++]);  //Send Data to Slave
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          };
      case 0x30:
          {
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            EndFlag=1;
            break;
          };
      case 0x8C: // MACTFlag+DFFlag+ACKFlag
          { /* DATA has been transmitted and ACK has been received */
            if(DataTxIndex<DataTxLen)
            {
              DrvI2C_WriteData(Sendbuf[DataTxIndex++]);  //Send Data to Slave
              DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            }
            else
            {
              if(I2C_RW == WRITE)
              {
                DrvI2C_Ctrl(0,1,0,0);  // I2C as master sends STOP signal
                EndFlag=1;
              }
              else if(I2C_RW == READ)
                DrvI2C_Ctrl(1,0,0,0);  // I2C as master sends START signal
              DataTxIndex=0;
            }
            break;
          };
      case 0x88: //MACTFlag+DFFlag
          { /* DATA has been transmitted and NACK has been received */
            DrvI2C_Ctrl(0,1,0,0);  // I2C as master sends STOP signal
            DataTxIndex=0;
            EndFlag=1;
            break;
          };
      case 0xB0:
          { /* A repeated START has been transmitted. */
            DrvI2C_WriteData(I2C_TARGET | READ);  //Send Slave Address & R/W Bit
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            break;
          }
      case 0x94: //MACTFlag+RWFlag
          { /* Slave A + R has been transmitted. ACK has been received. */
            DrvI2C_Ctrl(0,0,0,1);  // Set ACK bit
            break;
          };
      case 0x9C: //MACTFlag+RWFlag+DFFlag+ACKFlag
          { /* Data byte has been received. ACK has been transmitted. */
            if(DataRxLen>DataRxIndex)
            {
              Recbuf[DataRxIndex++]=DrvI2C_ReadData();
              DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            }
            else
            {
              Recbuf[DataRxIndex++]=DrvI2C_ReadData();
              DrvI2C_Ctrl(0,1,0,0);  // I2C as master sends STOP signal
              EndFlag=1;
            }
            break;
          };
      case 0x98: //MACTFlag+RWFlag+DFFlag
          { /* Data byte has been received. NACK has been transmitted. */
            DrvI2C_Ctrl(0,1,0,0);  // I2C as master sends STOP signal
            EndFlag=1;
            break;
          };
      default:
          {
            DrvI2C_Ctrl(0,0,0,0);  // Clear all I2C flag
            EndFlag=1;
            break;
          };
    }
    DrvI2C_ClearEIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CIF)
    SYS_EnableGIE(7,0x3F);       // Enable GIE(Global Interrupt)
  }
  if(DrvI2C_ReadIntFlag()==E_DRVI2C_ERROR_INT)  //Get I2C Error Interrupt Flag
  {
    EndFlag=1;
    DrvI2C_ClearIRQ();
    DrvI2C_ClearIntFlag(2); // Clear I2C Interrupt Flag(I2CEIF)
    DrvI2C_Ctrl(0,1,0,0);   // I2C as master sends STOP signal
    SYS_EnableGIE(7,0x3F);       // Enable GIE(Global Interrupt)
  }
}

/*---------------------------------------------------------------------------*/
/* Software Delay Subroutines                                                */
/*---------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
  for(;num>0;num--)
    asm("NOP");
}

/*---------------------------------------------------------------------------*/
/* End Of File                                                               */
/*---------------------------------------------------------------------------*/




void RTC_Initial(void)
{
  clk_00=0x0404;
  //RTC CLK;
  clk_08=0x8080ff00;
  //RTC KEY;
  rtc_00=0xff60ff00;

  DrvRTC_WriteEnable();
  Delay(0x01);
  //DrvRTC_ClockSource(E_INTERNAL_CLOCK);//35KHz
  DrvRTC_ClockSource(E_EXTERNAL_CLOCK);//32768Hz

  DrvRTC_PeriodicTimeEnable(7);//set 1/1
  DrvRTC_Enable();
  DrvRTC_HourFormat(0);

  Delay(0x01);
  S_DRVRTC_TIME_DATA_T sCurTime;//setting start
  DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
  sCurTime.u8cClockDisplay=0;
  sCurTime.u8cAmPm=0;
  sCurTime.u32cSecond=0;
  sCurTime.u32cMinute=15;
  sCurTime.u32cHour=18;
  sCurTime.u32cDayOfWeek=1;
  sCurTime.u32cDay=4;
  sCurTime.u32cMonth=11;
  sCurTime.u32Year=2013;
  sCurTime.u8IsEnableWakeUp=0;
  DrvRTC_Write(DRVRTC_CURRENT_TIME,&sCurTime);
  Delay(0x01);

  asm volatile("sethi $r0, 0xc0000");
  asm volatile("ori  $r0, $r0, 0x003f");
  asm volatile("mtsr $r0, $INT_MASK");
  asm volatile("movi $r0, 0x70009");
  asm volatile("mtsr $r0, $PSW");

}


/*---------------------------------------------------------------------------*/
void HW1_ISR(void)
{
  int_04=0xff20ff00;
  rtc_00=0xff60ff31;
  sss++;
  asm volatile("sethi $r0, 0xc0000");
  asm volatile("ori  $r0, $r0, 0x003f");
  asm volatile("mtsr $r0, $INT_MASK");
  asm volatile("movi $r0, 0x70009");
  asm volatile("mtsr $r0, $PSW");
}
/*---------------------------------------------------------------------------*/
void HW4_ISR(void)
{
	DrvGPIO_ClearIntFlag(E_PT1,0xc0);         //Clear PT1 interrupt flag
	  b=DrvGPIO_GetBit(E_PT1,6);
	  c=DrvGPIO_GetBit(E_PT1,7);

	  asm volatile("sethi $r0, 0xc0000");
	  asm volatile("ori  $r0, $r0, 0x003f");
	  asm volatile("mtsr $r0, $INT_MASK");
	  asm volatile("movi $r0, 0x70009");
	  asm volatile("mtsr $r0, $PSW");
}
/*---------------------------------------------------------------------------*/
void timeloop(void)
{
	  S_DRVRTC_TIME_DATA_T sCurTime;//read
	  DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
	  sec=sCurTime.u32cSecond;
	  min=sCurTime.u32cMinute;
	  hour=sCurTime.u32cHour;
	  week=sCurTime.u32cDayOfWeek;
	  day=sCurTime.u32cDay;
	  month=sCurTime.u32cMonth;
	  year=sCurTime.u32Year;
	  clk_00=0x0f05;
	  clk_08=0x0f00;
	  LCD_DATA_DISPLAY_1(hour);
	  LCD_DATA_DISPLAY_2(min);
	  LCD_DATA_DISPLAY_3(sec);
	  clk_08=0x0303;
	  clk_00=0x0100;
	  asm volatile("sethi $r0, 0xc0000");
	  asm volatile("ori  $r0, $r0, 0x003f");
	  asm volatile("mtsr $r0, $INT_MASK");
	  asm volatile("movi $r0, 0x70009");
	  asm volatile("mtsr $r0, $PSW");
}
/*---------------------------------------------------------------------------*/
void Hc(void)
{
	  while(1)
	  {
zero:	  sss=0;
		  ss=0;
		  mm=0;
		  LCD_DATA_DISPLAY_3(sss);
		  LCD_DATA_DISPLAY_2(ss);
		  LCD_DATA_DISPLAY_1(mm);
		  //b=DrvGPIO_GetBit(E_PT1,6);
		  //c=DrvGPIO_GetBit(E_PT1,7);
		  if(c==0) Pb_7();
		  if(pb7==1)
		  {
			  pb7=0;
			  break;
		  }
		  if(b==0) Pb_6();
		  if(pb6==1)
		  {
			  pb6=0;
			  sss=0;
			  while(1)
			  {
				  LCD_DATA_DISPLAY_3(sss);
				  LCD_DATA_DISPLAY_2(ss);
				  LCD_DATA_DISPLAY_1(mm);
				 // b=DrvGPIO_GetBit(E_PT1,6);
				 // c=DrvGPIO_GetBit(E_PT1,7);
				  if(sss>=128)
				  {
					  sss=0;
					  ss++;
					  if(ss>=60)
					  {
						  ss=0;
						  mm++;
					  }
				  }
				  if(c==0) Pb_7();
				  if(pb7==1)
				  {
					  pb7=0;
					  goto zero;
				  }
				  if(b==0) Pb_6();
				  if(pb6==1)
				  {
					  pb6=0;
					  stopsss=sss;
					  stopss=ss;
					  stopmm=mm;
					  Delay(0x300);
					  while(1)
					  {
						  LCD_DATA_DISPLAY_3(stopsss);
						  LCD_DATA_DISPLAY_2(stopss);
						  LCD_DATA_DISPLAY_1(stopmm);
						//  b=DrvGPIO_GetBit(E_PT1,6);
						 // c=DrvGPIO_GetBit(E_PT1,7);
						  if(c==0) Pb_7();
						  if(pb7==1)
						  {
							  pb7=0;
							  goto zero;
						  }
						  if(b==0) Pb_6();
						  if(pb6==1)
						  {
							  pb6=0;
							  sss=stopsss;
							  ss=stopss;
							  mm=stopmm;
							  break;
						  }
					  }
				  }
			  }
		  }

	  }
}
/*----------------------------------------------------------------------------*/
void loopday(void)
{
	while(1)
	{
		b=DrvGPIO_GetBit(E_PT1,6);
		c=DrvGPIO_GetBit(E_PT1,7);
		LCD_DATA_DISPLAY_4(year);
		//LCD_DATA_DISPLAY_1(month);
		//LCD_DATA_DISPLAY_2(day);
		//LCD_DATA_DISPLAY_3(week);
		if(c==0) Pb_7();
		if(pb7==1)
		{
			pb7=0;
			while(1)
			{
				LCD_DATA_DISPLAY_1(month);
				LCD_DATA_DISPLAY_2(day);
				LCD_DATA_DISPLAY_3(week);
				b=DrvGPIO_GetBit(E_PT1,6);
				c=DrvGPIO_GetBit(E_PT1,7);
				if(c==0) Pb_7();
				if(pb7==1)
				{
					pb7=0;
					goto bb;
				}
				if(b==0) Pb_6();
				if(pb6==1)
				{
					pb6=0;
					settime();
				}
			}
		}
		if(b==0) Pb_6();
		if(pb6==1)
		{
			pb6=0;
			setac();
		}
	}
	bb:   asm("NOP");
}
void setac(void)
{
	act=0;
	while(1)
	{
		b=DrvGPIO_GetBit(E_PT1,6);
		c=DrvGPIO_GetBit(E_PT1,7);
		LCD_DATA_DISPLAY_3(acs);
		LCD_DATA_DISPLAY_2(acm);
		LCD_DATA_DISPLAY_1(ach);
		if(c==0) Pb_7();
		if(pb7==1)
		{
			pb7=0;
			act++;
			if(act>=3) break;
		}
		if(b==0)
		{
			if(act==0)
			{
				acs++;
				if(acs>=60) acs=0;
			}
			if(act==1)
			{
				acm++;
				if(acm>=60) acm=0;
			}
			if(act==2)
			{
				ach++;
				if(ach>=24) ach=0;
			}

		}
	}
}
void settime(void)
{
	setyear=year;
	setmonth=month;
	setday=day;
	setweek=week;
	sethour=hour;
	setmin=min;
	setsec=sec;
	setnum=0;
	LCD_DATA_DISPLAY_4(setyear);
	while(1)
	{
		if(setnum==0) LCD_DATA_DISPLAY_4(setyear);
		else
		{
			if(setnum>=4)
			{
				LCD_DATA_DISPLAY_1(sethour);
				LCD_DATA_DISPLAY_2(setmin);
				LCD_DATA_DISPLAY_3(setsec);
			}
			 if(setnum<=3)
			{
				 LCD_DATA_DISPLAY_1(setmonth);
				 LCD_DATA_DISPLAY_2(setday);
				 LCD_DATA_DISPLAY_3(setweek);
			}
		}
		b=DrvGPIO_GetBit(E_PT1,6);
		c=DrvGPIO_GetBit(E_PT1,7);
		if(b==0) Pb_6();
		if(pb6==1)
		{
			pb6=0;
			if(setnum==0)
			{
				setyear++;
				if(setyear>=2100) setyear=2012;
			}
			if(setnum==1)
			{
				setmonth++;
				if(setmonth>=13) setmonth=1;
			}
			if(setnum==2)
			{
				setday++;
				if(setday>=32) setday=1;
			}
			if(setnum==3)
			{
				setweek++;
				if(setweek>=8) setweek=1;
			}
			if(setnum==4)
			{
				sethour++;
				if(sethour>=24) sethour=0;
			}
			if(setnum==5)
			{
				setmin++;
				if(setmin>=60) setmin=0;
			}
			if(setnum==6)
			{
				setsec++;
				if(setsec>=60) setsec=0;
			}
		}
		if(c==0) Pb_7();
		if(pb7==1)
		{
			pb7=0;
			setnum++;
			if(setnum==7)
			{
				setnum=0;
				S_DRVRTC_TIME_DATA_T sCurTime;//setting start
				DrvRTC_Read(DRVRTC_CURRENT_TIME,&sCurTime);
				sCurTime.u32cSecond=setsec;
				sCurTime.u32cMinute=setmin;
				sCurTime.u32cHour=sethour;
				sCurTime.u32cDayOfWeek=setweek;
				sCurTime.u32cDay=setday;
				sCurTime.u32cMonth=setmonth;
				sCurTime.u32Year=setyear;
				DrvRTC_Write(DRVRTC_CURRENT_TIME,&sCurTime);
				break;
			}

		}
	}
}

/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
void Pb_7()
{
	if(c==0)
	{
		pb7=1;
		c=1;
		while(1)
		{
			c=DrvGPIO_GetBit(E_PT1,7);
			if(c==1)break;
		}

	}
}
/*----------------------------------------------------------------------------*/
void Pb_6()
{
	if(b==0)
	{
		pb6=1;
		b=1;
		while(1)
		{
			b=DrvGPIO_GetBit(E_PT1,6);
			if(b==1)break;
		}

	}
}
/*----------------------------------------------------------------------------*/
void lookac(void)
{
	DrvGPIO_SetPortBits(E_PT1,0x00);
	if(hour==ach)
		if(min==acm)
			if(sec==acs) ac=1;
	if(ac==1)
	{
		DrvGPIO_SetPortBits(E_PT1,0x10);
		b=DrvGPIO_GetBit(E_PT1,6);
		c=DrvGPIO_GetBit(E_PT1,7);
		if(c==0) Pb_7();
		if(pb7==1)
		{
			pb7=0;
			ac=0;
		}
		if(b==0) Pb_6();
		if(pb6==1)
		{
			pb6=0;
			ac=0;
		}
	}
}






